function [] = createEnvironment

clear all;

% specify room size
room_length     = 6.0;
room_width      = 6.0;
room_height     = 3.0;

% specify texture of all walls, celing and floor
% 0 - no texture (white)
% 1 - concrete texture
% 2 - stones texture
floor       = 1;
ceiling     = 1;
north       = 2;
east        = 2;
south       = 2;
west        = 2;

% if no poles are desired, set NO_POLES = 1
NO_POLES = 0;

% OTHERWISE, specify radius and heights of each pole
radii       = [0.2 0.2 0.2 0.2 0.2];
tops        = [3.0 3.0 3.0 3.0 3.0];
bottoms     = [0.0 0.0 0.0 0.0 0.0];

% specify texture of all poles
% 0 - no texture (white)
% 1 - concrete texture
% 2 - stones texture
% 3 - orange
textures = [0 0 0 0 0];

% specify minimum distance between walls and poles 
margin = 0.5;

%% Open file that will be created
fileID = fopen('output.ac','wt');

%% Open template file that describes the walls, floor and ceiling
room = fileread('environment\WallFloorCeiling.ac');

%% Change template to describe room sizes and textures 
for i = 1:length(room)-10
    
    % fill in room sizes
    if room(i:i+5) == 'length'
        room(i:i+5) = num2str(double(room_length),'%#.5g');
    end
    if room(i:i+4) == 'width'
        room(i:i+4) = num2str(double(room_width),'%#.4g');
    end
    if room(i:i+5) == 'height'
        room(i:i+5) = num2str(double(room_height),'%#.5g');
    end
    
    % fill in room textures
    if room(i:i+6) == 'texture'
        
        if room(i+8:i+12) == 'floor'
            room = specifyTexture(i,floor,room);
        end
        
        if room(i+8:i+14) == 'ceiling'
            room = specifyTexture(i,ceiling,room);
        end
        
        if room(i+8:i+12) == 'north'
            room = specifyTexture(i,north,room);
        end
        
        if room(i+8:i+11) == 'east'
            room = specifyTexture(i,east,room);
        end
        
        if room(i+8:i+12) == 'south'
            room = specifyTexture(i,south,room);
        end
        
        if room(i+8:i+11) == 'west'
            room = specifyTexture(i,west,room);
        end
    end    
end

% write modified template to output file
fprintf(fileID,room);



%% Open template files that describes poles
pole1 = fileread('environment\pole1.ac');
pole2 = fileread('environment\pole2.ac');

%% Add poles to output file

if ~NO_POLES

    % create random pole locations
    N_POLES = min([length(radii) length(tops) length(bottoms)]);
    x = margin + (room_width-margin).*rand(N_POLES,1);
    y = margin + (room_length-margin).*rand(N_POLES,1);
    
    % combine templates for each pole and write into output file
    for i = 1:N_POLES
        
        poleHeader = pole1;
        poleFooter = pole2;
        
        % specify texture per pole
        for j = 1:length(poleHeader)-10
            if poleHeader(j:j+6) == 'texture'
                [poleHeader] = specifyTexturePoles(j,textures(i),poleHeader);                
            end
        end
        
        % specify color per pole 
        for j = 1:length(poleFooter)-10
            if poleFooter(j:j+2) == 'mat'
                [poleFooter] = specifyMaterialPoles(j,textures(i),poleFooter);                
            end
        end
        
        radius = radii(i);
        offset = radius*tan(360/16*pi/180);
        top = tops(i);
        bottom = bottoms(i);

        % poles are octagons that are defined individually
        fprintf(fileID,'\n\n');
        fprintf(fileID,poleHeader);
        fprintf(fileID,['\n' num2str(-offset+y(i)) ' ' num2str(bottom) ' ' num2str(radius+x(i))]);
        fprintf(fileID,['\n' num2str(offset+y(i)) ' ' num2str(bottom) ' ' num2str(radius+x(i))]);
        fprintf(fileID,['\n' num2str(offset+y(i)) ' ' num2str(top) ' ' num2str(radius+x(i))]);
        fprintf(fileID,['\n' num2str(-offset+y(i)) ' ' num2str(top) ' ' num2str(radius+x(i)) '\n']);
        fprintf(fileID,poleFooter);
        
        fprintf(fileID,'\n\n');
        fprintf(fileID,poleHeader);
        fprintf(fileID,['\n' num2str(offset+y(i)) ' ' num2str(bottom) ' ' num2str(radius+x(i))]);
        fprintf(fileID,['\n' num2str(radius+y(i)) ' ' num2str(bottom) ' ' num2str(offset+x(i))]);
        fprintf(fileID,['\n' num2str(radius+y(i)) ' ' num2str(top) ' ' num2str(offset+x(i))]);
        fprintf(fileID,['\n' num2str(offset+y(i)) ' ' num2str(top) ' ' num2str(radius+x(i)) '\n']);
        fprintf(fileID,poleFooter);
        
        fprintf(fileID,'\n\n');
        fprintf(fileID,poleHeader);
        fprintf(fileID,['\n' num2str(radius+y(i)) ' ' num2str(bottom) ' ' num2str(offset+x(i))]);
        fprintf(fileID,['\n' num2str(radius+y(i)) ' ' num2str(bottom) ' ' num2str(-offset+x(i))]);
        fprintf(fileID,['\n' num2str(radius+y(i)) ' ' num2str(top) ' ' num2str(-offset+x(i))]);
        fprintf(fileID,['\n' num2str(radius+y(i)) ' ' num2str(top) ' ' num2str(offset+x(i)) '\n']);
        fprintf(fileID,poleFooter);
        
        fprintf(fileID,'\n\n');
        fprintf(fileID,poleHeader);
        fprintf(fileID,['\n' num2str(radius+y(i)) ' ' num2str(bottom) ' ' num2str(-offset+x(i))]);
        fprintf(fileID,['\n' num2str(offset+y(i)) ' ' num2str(bottom) ' ' num2str(-radius+x(i))]);
        fprintf(fileID,['\n' num2str(offset+y(i)) ' ' num2str(top) ' ' num2str(-radius+x(i))]);
        fprintf(fileID,['\n' num2str(radius+y(i)) ' ' num2str(top) ' ' num2str(-offset+x(i)) '\n']);
        fprintf(fileID,poleFooter);
        
        fprintf(fileID,'\n\n');
        fprintf(fileID,poleHeader);
        fprintf(fileID,['\n' num2str(offset+y(i)) ' ' num2str(bottom) ' ' num2str(-radius+x(i))]);
        fprintf(fileID,['\n' num2str(-offset+y(i)) ' ' num2str(bottom) ' ' num2str(-radius+x(i))]);
        fprintf(fileID,['\n' num2str(-offset+y(i)) ' ' num2str(top) ' ' num2str(-radius+x(i))]);
        fprintf(fileID,['\n' num2str(offset+y(i)) ' ' num2str(top) ' ' num2str(-radius+x(i)) '\n']);
        fprintf(fileID,poleFooter);
        
        fprintf(fileID,'\n\n');
        fprintf(fileID,poleHeader);
        fprintf(fileID,['\n' num2str(-offset+y(i)) ' ' num2str(bottom) ' ' num2str(-radius+x(i))]);
        fprintf(fileID,['\n' num2str(-radius+y(i)) ' ' num2str(bottom) ' ' num2str(-offset+x(i))]);
        fprintf(fileID,['\n' num2str(-radius+y(i)) ' ' num2str(top) ' ' num2str(-offset+x(i))]);
        fprintf(fileID,['\n' num2str(-offset+y(i)) ' ' num2str(top) ' ' num2str(-radius+x(i)) '\n']);
        fprintf(fileID,poleFooter);
        
        fprintf(fileID,'\n\n');
        fprintf(fileID,poleHeader);
        fprintf(fileID,['\n' num2str(-radius+y(i)) ' ' num2str(bottom) ' ' num2str(offset+x(i))]);
        fprintf(fileID,['\n' num2str(-radius+y(i)) ' ' num2str(bottom) ' ' num2str(-offset+x(i))]);
        fprintf(fileID,['\n' num2str(-radius+y(i)) ' ' num2str(top) ' ' num2str(-offset+x(i))]);
        fprintf(fileID,['\n' num2str(-radius+y(i)) ' ' num2str(top) ' ' num2str(offset+x(i)) '\n']);
        fprintf(fileID,poleFooter);
        
        fprintf(fileID,'\n\n');
        fprintf(fileID,poleHeader);
        fprintf(fileID,['\n' num2str(-radius+y(i)) ' ' num2str(bottom) ' ' num2str(offset+x(i))]);
        fprintf(fileID,['\n' num2str(-offset+y(i)) ' ' num2str(bottom) ' ' num2str(radius+x(i))]);
        fprintf(fileID,['\n' num2str(-offset+y(i)) ' ' num2str(top) ' ' num2str(radius+x(i))]);
        fprintf(fileID,['\n' num2str(-radius+y(i)) ' ' num2str(top) ' ' num2str(offset+x(i)) '\n']);
        fprintf(fileID,poleFooter);

    end
    
end






fclose(fileID);

end

%% script for filling in texture definitions for walls, ceiling and floor
function [file] = specifyTexture(i,texture,file)

    if texture == 0 || texture == 3 || texture == 4
        file(i-2:i+14) = '                 ';
    end
    if texture == 1
        file(i:i+41) = ['texture "Concrete_Form_4x8.rgb"' char(10) 'texrep 7 7'];
    end
    if texture == 2
        file(i:i+45) = ['texture "multi_coloured_stones.rgb"' char(10) 'texrep 7 7'];
    end

end

%% script for filling in texture definitions for poles
function [file] = specifyTexturePoles(i,texture,file)

    if texture == 0 || texture == 3
        file(i-2:i+14) = '                 ';
    end
    if texture == 1
        file(i:i+41) = ['texture "Concrete_Form_4x8.rgb"' char(10) 'texrep 1 1'];
    end
    if texture == 2
        file(i:i+45) = ['texture "multi_coloured_stones.rgb"' char(10) 'texrep 1 1'];
    end

end

%% script for filling in color definitions for poles
function [file] = specifyMaterialPoles(i,texture,file)

    if texture < 3
        file(i+4) = '0';
    end
    if texture == 3
        file(i+4) = '1';
    end
    
end

